@@ -240,25 +240,40 @@ bool DSysInfoPrivate::ensureOsVersion()
     DDesktopEntry entry(OS_VERSION_FILE);
     bool ok = false;
 
+#define D_ASSET_EXIT(con, msg) do { \
+    if (!(con)) { \
+        qWarning() << __func__ << msg; \
+        return false; \
+    } \
+} while (false)
+
+    D_ASSET_EXIT(entry.status() == DDesktopEntry::NoError, entry.status());
+
     // 先获取版本信息
     // ABCDE.xyz
     QString osb = entry.stringValue("OsBuild", "Version");
     QStringList osbs = osb.split(".");
-    Q_ASSERT(osbs.size() == 2 && osbs.value(0).size() == 5);
+    ok = (osbs.size() == 2 && osbs.value(0).size() == 5);
+    D_ASSET_EXIT(ok, "OsBuild version invalid!");
+
     const QStringList &left = osbs.value(0).split(QString(), QString::SkipEmptyParts);
-    Q_ASSERT(left.size() == 5);
+    D_ASSET_EXIT(left.size() == 5, "OsBuild version(ls) invalid!");
+
     int idx = 0;
     osBuild.A = left.value(idx++, "0").toUInt(&ok);
-    Q_ASSERT(ok);
+    D_ASSET_EXIT(ok, "OsBuild version(a) invalid!");
     osBuild.B = left.value(idx++, "0").toUInt(&ok);
-    Q_ASSERT(ok);
+    D_ASSET_EXIT(ok, "OsBuild version(b) invalid!");
     osBuild.C = left.value(idx++, "0").toUInt(&ok);
-    if (!ok)
-         osBuild.C = uint(left.value(idx-1, "0").toLatin1().at(0));
+    if (!ok) {
+        auto c = left.value(idx-1, "0").toLatin1();
+        D_ASSET_EXIT(c.size()>0, "OsBuild version(c) invalid!");
+        osBuild.C = uint(c.at(0));
+    }
     osBuild.D = left.value(idx++, "0").toUInt(&ok);
-    Q_ASSERT(ok);
+    D_ASSET_EXIT(ok, "OsBuild version(d) invalid!");
     osBuild.E = left.value(idx++, "0").toUInt(&ok);
-    Q_ASSERT(ok);
+    D_ASSET_EXIT(ok, "OsBuild version(e) invalid!");
 
     // xyz
     osBuild.xyz = osbs.value(1).trimmed().toUInt(&ok);
